@@ -3,13 +3,19 @@ import os
 SCRIPT_PATH = os.path.dirname(os.path.realpath(__file__))
 
 
-def get_from_env_var_file(name, default=None):
+def get_config_var(name, default=None):
+    """
+    OS environment variable is checked first, if not found, check the env_var file.
+    """
     if not name:
         return False
-    with open(f"{SCRIPT_PATH}/env_var", "r", encoding="utf-8") as fh:
-        for line in fh.readlines():
-            if line.startswith(name):
-                return line.split("=", 1)[1].strip()
+    if os.getenv(name):
+        return os.getenv(name)
+    else:
+        with open(f"{SCRIPT_PATH}/env_var", "r", encoding="utf-8") as fh:
+            for line in fh.readlines():
+                if line.startswith(name):
+                    return line.split("=", 1)[1].strip()
     return default
 
 
@@ -28,11 +34,11 @@ NODE_MONITOR_INTERVAL_SEC = 10
 DEVICE_MONITOR_INTERVAL_SEC = 5
 STAT_COLLECTOR_INTERVAL_SEC = 60*5  # 5 minutes
 LVOL_STAT_COLLECTOR_INTERVAL_SEC = 5
-LVOL_MONITOR_INTERVAL_SEC = 60
+LVOL_MONITOR_INTERVAL_SEC = 30
 DEV_MONITOR_INTERVAL_SEC = 10
 DEV_STAT_COLLECTOR_INTERVAL_SEC = 5
 PROT_STAT_COLLECTOR_INTERVAL_SEC = 2
-SPDK_STAT_COLLECTOR_INTERVAL_SEC = 5
+SPDK_STAT_COLLECTOR_INTERVAL_SEC = 30
 DISTR_EVENT_COLLECTOR_INTERVAL_SEC = 2
 DISTR_EVENT_COLLECTOR_NUM_OF_EVENTS = 10
 CAP_MONITOR_INTERVAL_SEC = 10
@@ -70,11 +76,11 @@ TASK_EXEC_INTERVAL_SEC = 10
 TASK_EXEC_RETRY_COUNT = 8
 
 SIMPLY_BLOCK_SPDK_CORE_IMAGE = "simplyblock/spdk-core:v24.05-tag-latest"
-SIMPLY_BLOCK_DOCKER_IMAGE = get_from_env_var_file(
+SIMPLY_BLOCK_DOCKER_IMAGE = get_config_var(
         "SIMPLY_BLOCK_DOCKER_IMAGE","simplyblock/simplyblock:main")
-SIMPLY_BLOCK_CLI_NAME = get_from_env_var_file(
+SIMPLY_BLOCK_CLI_NAME = get_config_var(
         "SIMPLY_BLOCK_COMMAND_NAME", "sbcli")
-SIMPLY_BLOCK_SPDK_ULTRA_IMAGE = get_from_env_var_file(
+SIMPLY_BLOCK_SPDK_ULTRA_IMAGE = get_config_var(
         "SIMPLY_BLOCK_SPDK_ULTRA_IMAGE", "public.ecr.aws/simply-block/ultra:main-latest")
 
 GELF_PORT = 12202
@@ -114,14 +120,14 @@ INSTANCE_STORAGE_DATA = {
         'm6id.8xlarge': {'number_of_devices': 1, 'size_per_device_gb': 1900},
     }
 
-MAX_SNAP_COUNT = 15
+MAX_SNAP_COUNT = 100
 
 SPDK_PROXY_MULTI_THREADING_ENABLED=True
 SPDK_PROXY_TIMEOUT=60*5
-LVOL_NVME_CONNECT_RECONNECT_DELAY=3
-LVOL_NVME_CONNECT_CTRL_LOSS_TMO=-1
+LVOL_NVME_CONNECT_RECONNECT_DELAY=2
+LVOL_NVME_CONNECT_CTRL_LOSS_TMO=60*60
 LVOL_NVME_CONNECT_NR_IO_QUEUES=6
-LVOL_NVME_KEEP_ALIVE_TO=12
+LVOL_NVME_KEEP_ALIVE_TO=5
 LVOL_NVMF_PORT_START=9100
 QPAIR_COUNT=32
 NVME_TIMEOUT_US=20000000
@@ -132,3 +138,16 @@ HA_JM_COUNT=3
 SENTRY_SDK_DNS = "https://745047b017ac424b4173550e19910fb7@o4508953941311488.ingest.de.sentry.io/4508996361584720"
 ONE_KB = 1024
 TEMP_CORES_FILE = "/etc/simplyblock/tmp_cores_config"
+PROMETHEUS_MULTIPROC_DIR = "/etc/simplyblock/metrics"
+
+LINUX_DRV_MASS_STORAGE_ID = 1
+LINUX_DRV_MASS_STORAGE_NVME_TYPE_ID = 8
+
+NODE_NVMF_PORT_START=9060
+
+NODE_HUBLVOL_PORT_START=9030
+
+NODES_CONFIG_FILE = "/etc/simplyblock/sn_config_file"
+SYSTEM_INFO_FILE = "/etc/simplyblock/system_info"
+
+LVO_MAX_NAMESPACES_PER_SUBSYS=32
