@@ -1,7 +1,6 @@
 # coding=utf-8
 import logging as lg
 import json
-import string
 import random
 import sys
 import time
@@ -9,28 +8,17 @@ import uuid
 from datetime import datetime
 from typing import Tuple
 
-from simplyblock_core import utils, constants, distr_controller
-from simplyblock_core.controllers import snapshot_controller, pool_controller, lvol_events, caching_node_controller, \
-    tasks_controller
+from simplyblock_core import utils, constants
+from simplyblock_core.controllers import snapshot_controller, pool_controller, lvol_events, caching_node_controller
 from simplyblock_core.db_controller import DBController
-from simplyblock_core.models.nvme_device import NVMeDevice
 from simplyblock_core.models.pool import Pool
 from simplyblock_core.models.lvol_model import LVol
 from simplyblock_core.models.storage_node import StorageNode
 from simplyblock_core.rpc_client import RPCClient
-from simplyblock_core.snode_client import SNodeClient
 
 logger = lg.getLogger()
 
 
-def _generate_hex_string(length):
-    def _generate_string(length):
-        return ''.join(random.SystemRandom().choice(
-            string.ascii_letters + string.digits) for _ in range(length))
-
-    return _generate_string(length).encode('utf-8').hex()
-
-
 def _create_crypto_lvol(rpc_client, name, base_name, key1, key2):
     key_name = f'key_{name}'
     ret = rpc_client.lvol_crypto_key_create(key_name, key1, key2)
@@ -68,7 +56,7 @@ def ask_for_device_number(devices_list):
 
 
 def ask_for_lvol_vuid():
-    question = f"Enter VUID number: "
+    question = "Enter VUID number: "
     while True:
         sys.stdout.write(question)
         choice = str(input())
@@ -77,7 +65,7 @@ def ask_for_lvol_vuid():
             return ch
         except Exception as e:
             logger.debug(e)
-            sys.stdout.write(f"Please respond with numbers")
+            sys.stdout.write("Please respond with numbers")
 
 
 def validate_add_lvol_func(name, size, host_id_or_name, pool_id_or_name,
@@ -95,7 +83,7 @@ def validate_add_lvol_func(name, size, host_id_or_name, pool_id_or_name,
     #  host validation
     # snode = db_controller.get_storage_node_by_id(host_id_or_name)
     # if not snode:
-    #     snode = db_controller.get_storage_node_by_hostname(host_id_or_name)
+    #     snode = db_controller.get_storage_nodes_by_hostname(host_id_or_name)
     #     if not snode:
     #         return False, f"Can not find storage node: {host_id_or_name}"
 
@@ -131,38 +119,9 @@ def validate_add_lvol_func(name, size, host_id_or_name, pool_id_or_name,
             if lvol.lvol_name == name:
                 return False, f"LVol name must be unique: {name}"
 
-    if pool.has_qos():
-        if pool.max_rw_ios_per_sec > 0:
-            if max_rw_iops <= 0:
-                return False, "LVol must have max_rw_iops value because the Pool has it set"
-            total = pool_controller.get_pool_total_rw_iops(pool.get_id())
-            if max_rw_iops + total > pool.max_rw_ios_per_sec:
-                return False, f"Invalid LVol max_rw_iops: {max_rw_iops} " \
-                              f"Pool Max RW IOPS has reached {total} of {pool.max_rw_ios_per_sec}"
-
-        if pool.max_rw_mbytes_per_sec > 0:
-            if max_rw_mbytes <= 0:
-                return False, "LVol must have max_rw_mbytes value because the Pool has it set"
-            total = pool_controller.get_pool_total_rw_mbytes(pool.get_id())
-            if max_rw_mbytes + total > pool.max_rw_mbytes_per_sec:
-                return False, f"Invalid LVol max_rw_mbytes: {max_rw_mbytes} " \
-                              f"Pool Max RW MBytes has reached {total} of {pool.max_rw_mbytes_per_sec}"
-
-        if pool.max_r_mbytes_per_sec > 0:
-            if max_r_mbytes <= 0:
-                return False, "LVol must have max_r_mbytes value because the Pool has it set"
-            total = pool_controller.get_pool_total_r_mbytes(pool.get_id())
-            if max_r_mbytes + total > pool.max_r_mbytes_per_sec:
-                return False, f"Invalid LVol max_r_mbytes: {max_r_mbytes} " \
-                              f"Pool Max R MBytes has reached {total} of {pool.max_r_mbytes_per_sec}"
-
-        if pool.max_w_mbytes_per_sec > 0:
-            if max_w_mbytes <= 0:
-                return False, "LVol must have max_w_mbytes value because the Pool has it set"
-            total = pool_controller.get_pool_total_w_mbytes(pool.get_id())
-            if max_w_mbytes + total > pool.max_w_mbytes_per_sec:
-                return False, f"Invalid LVol max_w_mbytes: {max_w_mbytes} " \
-                              f"Pool Max W MBytes has reached {total} of {pool.max_w_mbytes_per_sec}"
+    # If user gave a QOS and the pool also have a QOS, return error
+    if (max_rw_iops or max_rw_mbytes or max_r_mbytes or max_w_mbytes) and (pool.has_qos()):
+        return False, "Both Lvol and Pool have QOS settings"
 
     return True, ""
 
@@ -308,12 +267,32 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     logger.info(f"Adding LVol: {name}")
     host_node = None
     if host_id_or_name:
-        host_node = db_controller.get_storage_node_by_id(host_id_or_name)
-        if not host_node:
-            host_node = db_controller.get_storage_node_by_hostname(host_id_or_name)
-            if not host_node:
+        try:
+            host_node = db_controller.get_storage_node_by_id(host_id_or_name)
+        except KeyError:
+            nodes = db_controller.get_storage_nodes_by_hostname(host_id_or_name)
+            if len(nodes) > 0:
+                host_node = nodes[0]
+            else:
                 return False, f"Can not find storage node: {host_id_or_name}"
 
+    if namespace:
+        master_lvol = db_controller.get_lvol_by_id(namespace)
+        if not master_lvol:
+            return False, f"LVol not found: {namespace}"
+
+        host_node = db_controller.get_storage_node_by_id(master_lvol.node_id)
+
+        lvols_count = 0
+        for lv in db_controller.get_lvols(host_node.cluster_id):
+            if lv.namespace == namespace:
+                lvols_count += 1
+
+        if lvols_count >= constants.LVO_MAX_NAMESPACES_PER_SUBSYS:
+            msg = f"Max namespaces reached: {lvols_count}"
+            logger.error(msg)
+            return False, msg
+
     pool = None
     for p in db_controller.get_pools():
         if pool_id_or_name == p.get_id() or pool_id_or_name == p.pool_name:
@@ -418,7 +397,6 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     lvol = LVol()
     lvol.lvol_name = name
     lvol.pvc_name = pvc_name or ""
-    lvol.namespace = namespace or ""
     lvol.size = int(size)
     lvol.max_size = int(max_size)
     lvol.status = LVol.STATUS_IN_CREATION
@@ -427,7 +405,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     lvol.ha_type = ha_type
     lvol.bdev_stack = []
     lvol.uuid = uid or str(uuid.uuid4())
-    lvol.guid = _generate_hex_string(16)
+    lvol.guid = utils.generate_hex_string(16)
     lvol.vuid = vuid
     lvol.lvol_bdev = f"LVOL_{vuid}"
 
@@ -436,16 +414,22 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
 
     lvol.mode = 'read-write'
     lvol.lvol_type = 'lvol'
-    lvol.nqn = cl.nqn + ":lvol:" + lvol.uuid
     lvol.lvol_priority_class = lvol_priority_class
 
+    if namespace:
+        master_lvol = db_controller.get_lvol_by_id(namespace)
+        lvol.nqn = master_lvol.nqn
+        lvol.namespace = namespace or ""
+    else:
+        lvol.nqn = cl.nqn + ":lvol:" + lvol.uuid
+
     nodes = []
     if host_node:
         nodes.insert(0, host_node)
     else:
         nodes = _get_next_3_nodes(cl.get_id(), lvol.size)
         if not nodes:
-            return False, f"No nodes found with enough resources to create the LVol"
+            return False, "No nodes found with enough resources to create the LVol"
         host_node = nodes[0]
 
     lvol.hostname = host_node.hostname
@@ -478,7 +462,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     lvol.bdev_stack = [lvol_dict]
 
     if use_crypto:
-        if crypto_key1 == None or crypto_key2 == None:
+        if crypto_key1 is None or crypto_key2 is None:
             return False, "encryption keys for lvol not provided"
         else:
             success, err = validate_aes_xts_keys(crypto_key1, crypto_key2)
@@ -529,7 +513,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
             if is_node_leader(host_node, lvol.lvs_name):
                 primary_node = host_node
                 if sec_node.status == StorageNode.STATUS_DOWN:
-                    msg = f"Secondary node is in down status, can not create lvol"
+                    msg = "Secondary node is in down status, can not create lvol"
                     logger.error(msg)
                     lvol.remove(db_controller.kv_store)
                     return False, msg
@@ -557,7 +541,7 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
 
         else:
             # both primary and secondary are not online
-            msg = f"Host nodes are not online"
+            msg = "Host nodes are not online"
             logger.error(msg)
             lvol.remove(db_controller.kv_store)
             return False, msg
@@ -593,6 +577,8 @@ def add_lvol_ha(name, size, host_id_or_name, ha_type, pool_id_or_name, use_comp,
     lvol.write_to_db(db_controller.kv_store)
     lvol_events.lvol_create(lvol)
 
+    connect_lvol_to_pool(lvol.uuid)
+
     # set QOS
     if max_rw_iops or max_rw_mbytes or max_r_mbytes or max_w_mbytes:
         set_lvol(lvol.uuid, max_rw_iops, max_rw_mbytes, max_r_mbytes, max_w_mbytes)
@@ -653,52 +639,54 @@ def _create_bdev_stack(lvol, snode, is_primary=True):
 
 def add_lvol_on_node(lvol, snode, is_primary=True):
     rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
-    db_controller = DBController()
 
     ret, msg = _create_bdev_stack(lvol, snode, is_primary=is_primary)
     if not ret:
         return False, msg
 
-    if is_primary:
-        min_cntlid = 1
-    else:
-        min_cntlid =  1000
-    logger.info("creating subsystem %s", lvol.nqn)
-    ret = rpc_client.subsystem_create(lvol.nqn, 'sbcli-cn', lvol.uuid, min_cntlid)
-    logger.debug(ret)
+    if not lvol.namespace:
+        if is_primary:
+            min_cntlid = 1
+        else:
+            min_cntlid =  1000
+        logger.info("creating subsystem %s", lvol.nqn)
+        ret = rpc_client.subsystem_create(lvol.nqn, lvol.ha_type, lvol.uuid, min_cntlid,
+                                          max_namespaces=constants.LVO_MAX_NAMESPACES_PER_SUBSYS)
 
-    # add listeners
-    logger.info("adding listeners")
-    for iface in snode.data_nics:
-        if iface.ip4_address:
-            tr_type = iface.get_transport_type()
-            logger.info("adding listener for %s on IP %s" % (lvol.nqn, iface.ip4_address))
-            ret = rpc_client.listeners_create(lvol.nqn, tr_type, iface.ip4_address, lvol.subsys_port)
-            if not ret:
-                return False, f"Failed to create listener for {lvol.get_id()}"
-            is_optimized = False
-            if lvol.node_id == snode.get_id():
-                is_optimized = True
-            logger.info(f"Setting ANA state: {is_optimized}")
-            ret = rpc_client.nvmf_subsystem_listener_set_ana_state(
-                lvol.nqn, iface.ip4_address, lvol.subsys_port, is_optimized)
-            if not ret:
-                return False, f"Failed to set ANA state for {lvol.get_id()}"
+        ana_state = "non_optimized"
+        if lvol.node_id == snode.get_id():
+            ana_state = "optimized"
+
+        # add listeners
+        logger.info("adding listeners")
+        for iface in snode.data_nics:
+            if iface.ip4_address:
+                tr_type = iface.get_transport_type()
+                logger.info("adding listener for %s on IP %s" % (lvol.nqn, iface.ip4_address))
+                ret, err = rpc_client.nvmf_subsystem_add_listener(
+                    lvol.nqn, tr_type, iface.ip4_address, lvol.subsys_port, ana_state)
+                if not ret:
+                    if err and "code" in err and err["code"] == -32602:
+                        logger.warning("listener already exists")
+                    else:
+                        return False, f"Failed to create listener for {lvol.get_id()}"
 
     logger.info("Add BDev to subsystem")
     ret = rpc_client.nvmf_subsystem_add_ns(lvol.nqn, lvol.top_bdev, lvol.uuid, lvol.guid)
     if not ret:
         return False, "Failed to add bdev to subsystem"
+    lvol.ns_id = int(ret)
 
     spdk_mem_info_after = rpc_client.ultra21_util_get_malloc_stats()
     logger.debug("ultra21_util_get_malloc_stats:")
     logger.debug(spdk_mem_info_after)
 
     ret = rpc_client.get_bdevs(f"{lvol.lvs_name}/{lvol.lvol_bdev}")
-    lvol_bdev = ret[0]
-
-    return lvol_bdev, None
-
+    if ret:
+        lvol_bdev = ret[0]
+        return lvol_bdev, None
+    else:
+        return False, "Failed to get lvol bdev"
 
 def is_node_leader(snode, lvs_name):
     rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
@@ -723,7 +711,8 @@ def recreate_lvol_on_node(lvol, snode, ha_inode_self=0, ana_state=None):
 
     min_cntlid = 1 + 1000 * ha_inode_self
     logger.info("creating subsystem %s", lvol.nqn)
-    rpc_client.subsystem_create(lvol.nqn, 'sbcli-cn', lvol.uuid, min_cntlid)
+    rpc_client.subsystem_create(lvol.nqn, lvol.ha_type, lvol.uuid, min_cntlid,
+                                max_namespaces=constants.LVO_MAX_NAMESPACES_PER_SUBSYS)
 
     # if namespace_found is False:
     logger.info("Add BDev to subsystem")
@@ -806,27 +795,33 @@ def _remove_bdev_stack(bdev_stack, rpc_client):
             logger.error(f"Failed to delete BDev {name}")
 
         bdev['status'] = 'deleted'
-        time.sleep(1)
     return True
 
 
 def delete_lvol_from_node(lvol_id, node_id, clear_data=True):
     db_controller = DBController()
     lvol = db_controller.get_lvol_by_id(lvol_id)
-    snode = db_controller.get_storage_node_by_id(node_id)
-    if not lvol or not snode:
+    if not lvol:
+        return True
+    try:
+        snode = db_controller.get_storage_node_by_id(node_id)
+    except KeyError:
         return True
 
     logger.info(f"Deleting LVol:{lvol.get_id()} from node:{snode.get_id()}")
-    rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password)
+    rpc_client = RPCClient(snode.mgmt_ip, snode.rpc_port, snode.rpc_username, snode.rpc_password, timeout=5, retry=2)
 
+    subsystem = rpc_client.subsystem_list(lvol.nqn)
     # 1- remove subsystem
-    if rpc_client.subsystem_list(lvol.nqn):
-        logger.info(f"Removing subsystem")
-        rpc_client.subsystem_delete(lvol.nqn)
+    if subsystem:
+        if len(subsystem[0]["namespaces"]) > 1:
+            rpc_client.nvmf_subsystem_remove_ns(lvol.nqn, lvol.ns_id)
+        else:
+            logger.info("Removing subsystem")
+            rpc_client.subsystem_delete(lvol.nqn)
 
     # 2- remove bdevs
-    logger.info(f"Removing bdev stack")
+    logger.info("Removing bdev stack")
     ret = _remove_bdev_stack(lvol.bdev_stack[::-1], rpc_client)
     if not ret:
         return False
@@ -845,15 +840,20 @@ def delete_lvol(id_or_name, force_delete=False):
             logger.error(f"lvol not found: {id_or_name}")
             return False
 
+    if lvol.status == LVol.STATUS_IN_DELETION:
+        logger.info(f"lvol:{lvol.get_id()} status is in deletion")
+        if not force_delete:
+            return False
+
     pool = db_controller.get_pool_by_id(lvol.pool_uuid)
     if pool.status == Pool.STATUS_INACTIVE:
-        logger.error(f"Pool is disabled")
+        logger.error("Pool is disabled")
         return False
 
     logger.debug(lvol)
-    snode = db_controller.get_storage_node_by_id(lvol.node_id)
-
-    if not snode:
+    try:
+        snode = db_controller.get_storage_node_by_id(lvol.node_id)
+    except KeyError:
         logger.error(f"lvol node id not found: {lvol.node_id}")
         if not force_delete:
             return False
@@ -873,13 +873,6 @@ def delete_lvol(id_or_name, force_delete=False):
         logger.info("Done")
         return True
 
-    # creating RPCClient instance
-    rpc_client = RPCClient(
-        snode.mgmt_ip,
-        snode.rpc_port,
-        snode.rpc_username,
-        snode.rpc_password)
-
     # disconnect from caching nodes:
     cnodes = db_controller.get_caching_nodes()
     for cnode in cnodes:
@@ -903,12 +896,6 @@ def delete_lvol(id_or_name, force_delete=False):
         sec_node = db_controller.get_storage_node_by_id(snode.secondary_node_id)
         host_node = db_controller.get_storage_node_by_id(snode.get_id())
 
-        sec_rpc_client = RPCClient(
-            sec_node.mgmt_ip,
-            sec_node.rpc_port,
-            sec_node.rpc_username,
-            sec_node.rpc_password, timeout=5, retry=1)
-
         primary_node = None
         secondary_node = None
         if host_node.status == StorageNode.STATUS_ONLINE:
@@ -916,7 +903,7 @@ def delete_lvol(id_or_name, force_delete=False):
             if is_node_leader(host_node, lvol.lvs_name):
                 primary_node = host_node
                 if sec_node.status == StorageNode.STATUS_DOWN:
-                    msg = f"Secondary node is in down status, can not delete lvol"
+                    msg = "Secondary node is in down status, can not delete lvol"
                     logger.error(msg)
                     return False, msg
                 elif sec_node.status == StorageNode.STATUS_ONLINE:
@@ -945,10 +932,21 @@ def delete_lvol(id_or_name, force_delete=False):
 
         else:
             # both primary and secondary are not online
-            msg = f"Host nodes are not online"
+            msg = "Host nodes are not online"
             logger.error(msg)
             return False, msg
 
+        # 1- delete subsystem from secondary
+        if secondary_node:
+            secondary_node = db_controller.get_storage_node_by_id(secondary_node.get_id())
+            if secondary_node.status == StorageNode.STATUS_ONLINE:
+                logger.info(f"Deleting subsystem for lvol:{lvol.get_id()} from node:{secondary_node.get_id()}")
+                secondary_rpc_client = secondary_node.rpc_client()
+                ret = secondary_rpc_client.subsystem_delete(lvol.nqn)
+                if not ret:
+                    logger.warning(f"Failed to delete subsystem from node: {secondary_node.get_id()}")
+
+        # 2- delete subsystem and lvol bdev from primary
         if primary_node:
 
             ret = delete_lvol_from_node(lvol.get_id(), primary_node.get_id())
@@ -957,6 +955,7 @@ def delete_lvol(id_or_name, force_delete=False):
                 if not force_delete:
                     return False
 
+        # 3- delete lvol bdev from secondary
         if secondary_node:
             secondary_node = db_controller.get_storage_node_by_id(secondary_node.get_id())
             if secondary_node.status == StorageNode.STATUS_ONLINE:
@@ -978,16 +977,47 @@ def delete_lvol(id_or_name, force_delete=False):
     if lvol.cloned_from_snap:
         snap = db_controller.get_snapshot_by_id(lvol.cloned_from_snap)
         if snap and snap.deleted is True:
-            lvols_count = 0
-            for lvol in db_controller.get_lvols():  # pass
-                if lvol.cloned_from_snap == snap.get_id():
-                    lvols_count += 1
-            if lvols_count == 0:
-                snapshot_controller.delete(snap.get_id())
+            snapshot_controller.delete(snap.get_id())
 
     logger.info("Done")
     return True
 
+def connect_lvol_to_pool(uuid):
+    db_controller = DBController()
+    lvol = db_controller.get_lvol_by_id(uuid)
+    if not lvol:
+        logger.error(f"lvol not found: {uuid}")
+        return False
+    pool = db_controller.get_pool_by_id(lvol.pool_uuid)
+    if pool.status == Pool.STATUS_INACTIVE:
+        logger.error("Pool is disabled")
+        return False
+
+    snode = db_controller.get_storage_node_by_id(lvol.node_id)
+    # creating RPCClient instance
+    rpc_client = RPCClient(
+        snode.mgmt_ip,
+        snode.rpc_port,
+        snode.rpc_username,
+        snode.rpc_password)
+
+    ret = rpc_client.bdev_lvol_add_to_group(pool.numeric_id, [lvol.top_bdev])
+    if not ret:
+        logger.error("RPC failed bdev_lvol_add_to_group")
+        return False
+
+    # re-apply the QOS limits
+    ret = rpc_client.bdev_lvol_set_qos_limit(pool.numeric_id, pool.max_rw_ios_per_sec,
+                                        pool.max_rw_mbytes_per_sec, pool.max_r_mbytes_per_sec,
+                                        pool.max_w_mbytes_per_sec)
+    if not ret:
+        logger.error("RPC failed bdev_set_qos_limit")
+        return False
+
+    lvol.write_to_db(db_controller.kv_store)
+    pool.write_to_db(db_controller.kv_store)
+    logger.info("Done")
+    return True
 
 def set_lvol(uuid, max_rw_iops, max_rw_mbytes, max_r_mbytes, max_w_mbytes, name=None):
     db_controller = DBController()
@@ -997,13 +1027,16 @@ def set_lvol(uuid, max_rw_iops, max_rw_mbytes, max_r_mbytes, max_w_mbytes, name=
         return False
     pool = db_controller.get_pool_by_id(lvol.pool_uuid)
     if pool.status == Pool.STATUS_INACTIVE:
-        logger.error(f"Pool is disabled")
+        logger.error("Pool is disabled")
+        return False
+    if pool.has_qos():
+        logger.error("Pool already has QOS settings")
         return False
 
     if name:
         lvol.lvol_name = name
 
-    snode = db_controller.get_storage_node_by_hostname(lvol.hostname)
+    snode = db_controller.get_storage_node_by_id(lvol.node_id)
     # creating RPCClient instance
     rpc_client = RPCClient(
         snode.mgmt_ip,
@@ -1079,6 +1112,7 @@ def list_lvols(is_json, cluster_id, pool_id_or_name, all=False):
             "Status": lvol.status,
             "IO Err": lvol.io_error,
             "Health": lvol.health_check,
+            "NS ID": lvol.ns_id,
         })
 
     if is_json:
@@ -1135,7 +1169,7 @@ def get_lvol(lvol_id_or_name, is_json):
         return utils.print_table(data2)
 
 
-def connect_lvol(uuid):
+def connect_lvol(uuid, ctrl_loss_tmo=constants.LVOL_NVME_CONNECT_CTRL_LOSS_TMO):
     db_controller = DBController()
     lvol = db_controller.get_lvol_by_id(uuid)
     if not lvol:
@@ -1162,11 +1196,11 @@ def connect_lvol(uuid):
                 "port": port,
                 "nqn": lvol.nqn,
                 "reconnect-delay": constants.LVOL_NVME_CONNECT_RECONNECT_DELAY,
-                "ctrl-loss-tmo": constants.LVOL_NVME_CONNECT_CTRL_LOSS_TMO,
+                "ctrl-loss-tmo": ctrl_loss_tmo,
                 "nr-io-queues": constants.LVOL_NVME_CONNECT_NR_IO_QUEUES,
                 "keep-alive-tmo": constants.LVOL_NVME_KEEP_ALIVE_TO,
                 "connect": f"sudo nvme connect --reconnect-delay={constants.LVOL_NVME_CONNECT_RECONNECT_DELAY} "
-                           f"--ctrl-loss-tmo={constants.LVOL_NVME_CONNECT_CTRL_LOSS_TMO} "
+                           f"--ctrl-loss-tmo={ctrl_loss_tmo} "
                            f"--nr-io-queues={constants.LVOL_NVME_CONNECT_NR_IO_QUEUES} "
                            f"--keep-alive-tmo={constants.LVOL_NVME_KEEP_ALIVE_TO} "
                            f"--transport={transport} --traddr={ip} --trsvcid={port} --nqn={lvol.nqn}",
@@ -1189,7 +1223,7 @@ def resize_lvol(id, new_size):
         return False, msg
 
     if lvol.size >= new_size:
-        msg = f"New size {new_size} must be higher than the original size {lvol.size}"
+        msg = f"New size {utils.humanbytes(new_size)} must be higher than the original size {utils.humanbytes(lvol.size)}"
         logger.error(msg)
         return False, msg
 
@@ -1200,7 +1234,7 @@ def resize_lvol(id, new_size):
 
     if 0 < pool.lvol_max_size < new_size:
         msg = f"Pool Max LVol size is: {utils.humanbytes(pool.lvol_max_size)}, "\
-              "LVol size: {utils.humanbytes(new_size)} must be below this limit"
+              f"LVol size: {utils.humanbytes(new_size)} must be below this limit"
         logger.error(msg)
         return False, msg
 
@@ -1239,7 +1273,7 @@ def resize_lvol(id, new_size):
             if is_node_leader(host_node, lvol.lvs_name):
                 primary_node = host_node
                 if sec_node.status == StorageNode.STATUS_DOWN:
-                    msg = f"Secondary node is in down status, can not resize lvol"
+                    msg = "Secondary node is in down status, can not resize lvol"
                     logger.error(msg)
                     return False, msg
 
@@ -1269,7 +1303,7 @@ def resize_lvol(id, new_size):
 
         else:
             # both primary and secondary are not online
-            msg = f"Host nodes are not online"
+            msg = "Host nodes are not online"
             logger.error(msg)
             return False, msg
 
@@ -1318,12 +1352,12 @@ def set_read_only(id):
 
     pool = db_controller.get_pool_by_id(lvol.pool_uuid)
     if pool.status == Pool.STATUS_INACTIVE:
-        logger.error(f"Pool is disabled")
+        logger.error("Pool is disabled")
         return False
 
     logger.info(f"Setting LVol: {lvol.get_id()} read only")
 
-    snode = db_controller.get_storage_node_by_hostname(lvol.hostname)
+    snode = db_controller.get_storage_node_by_id(lvol.node_id)
 
     # creating RPCClient instance
     rpc_client = RPCClient(
@@ -1373,6 +1407,8 @@ def get_capacity(lvol_uuid, history, records_count=20, parse_sizes=True):
         "size_used",
         "size_free",
         "size_util",
+        "size_prov",
+        "size_prov_util"
     ]
     new_records = utils.process_records(records_list, records_count, keys=cap_stats_keys)
 
@@ -1574,7 +1610,7 @@ def inflate_lvol(lvol_id):
         return False
     pool = db_controller.get_pool_by_id(lvol.pool_uuid)
     if pool.status == Pool.STATUS_INACTIVE:
-        logger.error(f"Pool is disabled")
+        logger.error("Pool is disabled")
         return False
 
     logger.info(f"Inflating LVol: {lvol.get_id()}")
