@@ -12,6 +12,7 @@ from simplyblock_core import utils
 from simplyblock_core.controllers import pool_events
 from simplyblock_core.db_controller import DBController
 from simplyblock_core.models.pool import Pool
+from simplyblock_core.rpc_client import RPCClient
 
 logger = lg.getLogger()
 
@@ -27,13 +28,15 @@ def add_pool(name, pool_max, lvol_max, max_rw_iops, max_rw_mbytes, max_r_mbytes,
         logger.error("Pool name is empty!")
         return False
 
-    for p in db_controller.get_pools():
+    pool_list = db_controller.get_pools()
+    for p in pool_list:
         if p.pool_name == name:
             logger.error(f"Pool found with the same name: {name}")
             return False
 
-    cluster = db_controller.get_cluster_by_id(cluster_id)
-    if not cluster:
+    try:
+        cluster = db_controller.get_cluster_by_id(cluster_id)
+    except KeyError:
         logger.error(f"Cluster not found: {cluster_id}")
         return False
 
@@ -53,6 +56,7 @@ def add_pool(name, pool_max, lvol_max, max_rw_iops, max_rw_mbytes, max_r_mbytes,
     pool = Pool()
     pool.uuid = str(uuid.uuid4())
     pool.cluster_id = cluster.get_id()
+    pool.numeric_id = _generate_numeric_id(pool_list)
     pool.pool_name = name
     if has_secret:
         pool.secret = _generate_string(20)
@@ -64,60 +68,102 @@ def add_pool(name, pool_max, lvol_max, max_rw_iops, max_rw_mbytes, max_r_mbytes,
     pool.max_w_mbytes_per_sec = max_w_mbytes
     pool.status = "active"
     pool.write_to_db(db_controller.kv_store)
-
     pool_events.pool_add(pool)
     logger.info("Done")
     return pool.get_id()
 
+def _generate_numeric_id(pool_list: list[Pool]):
+    if (pool_list is None) or (len(pool_list) == 0):
+        return 1
+
+    existing_ids = []
+    for p in pool_list:
+        existing_ids.append(p.numeric_id)
+
+    return (max(existing_ids) + 1)
+
 
 def set_pool_value_if_above(pool, key, value):
     logger.info(f"Updating pool {key}: {value}")
     current_value = getattr(pool, key)
     if value > current_value:
         setattr(pool, key, value)
-        return True
+    elif value == -1:
+        setattr(pool, key, 0)
     else:
-        logger.error(f"{key}: {value} can't be less than current value: {current_value}")
-        return False
+        msg = f"{key}: {value} can't be less than current value: {current_value}"
+        logger.error(msg)
+        return False, msg
+    return True, None
 
+def qos_exists_on_child_lvol(db_controller: DBController, pool_uuid):
+    for lvol in db_controller.get_lvols_by_pool_id(pool_uuid):
+        if lvol.has_qos():
+            return True
+    return False
 
-def set_pool(uuid, pool_max, lvol_max, max_rw_iops,
-             max_rw_mbytes, max_r_mbytes, max_w_mbytes):
+def set_pool(uuid, pool_max=0, lvol_max=0, max_rw_iops=0,
+             max_rw_mbytes=0, max_r_mbytes=0, max_w_mbytes=0, name=""):
     db_controller = DBController()
     pool = db_controller.get_pool_by_id(uuid)
     if not pool:
-        logger.error(f"Pool not found {uuid}")
-        return False
+        msg = f"Pool not found: {uuid}"
+        logger.error(msg)
+        return False, msg
 
     if pool.status == Pool.STATUS_INACTIVE:
-        logger.error("Pool is disabled")
-        return False
+        msg = "Pool is disabled"
+        logger.error(msg)
+        return False, msg
 
-    if pool_max:
-        if not set_pool_value_if_above(pool, "pool_max_size", pool_max):
-            return False
-    if lvol_max:
-        if not set_pool_value_if_above(pool, "lvol_max_size", lvol_max):
-            return False
+    if name and name != pool.pool_name:
+        for p in db_controller.get_pools():
+            if p.pool_name == name:
+                msg = f"Pool found with the same name: {name}"
+                logger.error(msg)
+                return False, msg
+        pool.pool_name = name
 
-    if max_rw_iops:
-        if not set_pool_value_if_above(pool, "max_rw_ios_per_sec", max_rw_iops):
-            return False
-    if max_rw_mbytes:
-        if not set_pool_value_if_above(pool, "max_rw_mbytes_per_sec", max_rw_mbytes):
-            return False
-    if max_r_mbytes:
-        if not set_pool_value_if_above(pool, "max_r_mbytes_per_sec", max_r_mbytes):
-            return False
-    if max_w_mbytes:
-        if not set_pool_value_if_above(pool, "max_w_mbytes_per_sec", max_w_mbytes):
-            return False
+    # Normalize inputs
+    max_rw_iops = max_rw_iops or 0
+    max_rw_mbytes = max_rw_mbytes or 0
+    max_r_mbytes = max_r_mbytes or 0
+    max_w_mbytes = max_w_mbytes or 0
+
+    # Check for QoS conflict
+    if (max_rw_iops + max_rw_mbytes + max_r_mbytes + max_w_mbytes) > 0:
+        if qos_exists_on_child_lvol(db_controller, uuid):
+            logger.error("One of the lvols already has QOS")
+            return False, "QOS already set on one of the lvols"
+
+    # Update values if needed
+    fields_to_update = [
+        ("pool_max_size", pool_max),
+        ("lvol_max_size", lvol_max),
+        ("max_rw_ios_per_sec", max_rw_iops),
+        ("max_rw_mbytes_per_sec", max_rw_mbytes),
+        ("max_r_mbytes_per_sec", max_r_mbytes),
+        ("max_w_mbytes_per_sec", max_w_mbytes),
+    ]
+
+    for key, val in fields_to_update:
+        if val:
+            success, err = set_pool_value_if_above(pool, key, val)
+            if err:
+                return False, err
+
+    # Apply QoS settings via RPC
+    for hostname in db_controller.get_hostnames_by_pool_id(uuid):
+        for sn in db_controller.get_storage_nodes_by_hostname(hostname):
+            client = RPCClient(sn.mgmt_ip, sn.rpc_port, sn.rpc_username, sn.rpc_password)
+            if not client.bdev_lvol_set_qos_limit(pool.numeric_id, max_rw_iops, max_rw_mbytes, max_r_mbytes, max_w_mbytes):
+                logger.error("RPC failed bdev_lvol_set_qos_limit")
+                return False, "RPC failed"
 
     pool.write_to_db(db_controller.kv_store)
     pool_events.pool_updated(pool)
     logger.info("Done")
-    return True
-
+    return True, None
 
 def delete_pool(uuid):
     db_controller = DBController()
@@ -128,7 +174,7 @@ def delete_pool(uuid):
         logger.error(f"Pool not found {uuid}")
         return False
     if pool.status == Pool.STATUS_INACTIVE:
-        logger.error(f"Pool is disabled")
+        logger.error("Pool is disabled")
         return False
 
     lvols = db_controller.get_lvols_by_pool_id(uuid)
