@@ -1,20 +1,17 @@
 # coding=utf-8
-import datetime
-import json
 import logging as lg
 import time
 import uuid
 
 from simplyblock_core.controllers import lvol_controller, snapshot_events, pool_controller
 
-from simplyblock_core import utils, distr_controller, constants
+from simplyblock_core import utils, constants
 from simplyblock_core.db_controller import DBController
 from simplyblock_core.models.pool import Pool
 from simplyblock_core.models.snapshot import SnapShot
 from simplyblock_core.models.lvol_model import LVol
 from simplyblock_core.models.storage_node import StorageNode
 from simplyblock_core.rpc_client import RPCClient
-from simplyblock_core.snode_client import SNodeClient
 
 
 logger = lg.getLogger()
@@ -31,7 +28,7 @@ def add(lvol_id, snapshot_name):
 
     pool = db_controller.get_pool_by_id(lvol.pool_uuid)
     if pool.status == Pool.STATUS_INACTIVE:
-        msg = f"Pool is disabled"
+        msg = "Pool is disabled"
         logger.error(msg)
         return False, msg
 
@@ -84,7 +81,8 @@ def add(lvol_id, snapshot_name):
     if cluster.status not in [cluster.STATUS_ACTIVE, cluster.STATUS_DEGRADED]:
         return False, f"Cluster is not active, status: {cluster.status}"
 
-    snap_bdev_name = f"SNAP_{utils.get_random_vuid()}"
+    snap_vuid = utils.get_random_snapshot_vuid()
+    snap_bdev_name = f"SNAP_{snap_vuid}"
     size = lvol.size
     blobid = 0
     snap_uuid = ""
@@ -102,7 +100,7 @@ def add(lvol_id, snapshot_name):
             if snap_bdev:
                 snap_uuid = snap_bdev[0]['uuid']
                 blobid = snap_bdev[0]['driver_specific']['lvol']['blobid']
-                cluster_size = cluster.distr_ndcs * cluster.page_size_in_blocks
+                cluster_size = cluster.page_size_in_blocks
                 num_allocated_clusters = snap_bdev[0]["driver_specific"]["lvol"]["num_allocated_clusters"]
                 used_size = int(num_allocated_clusters*cluster_size)
         else:
@@ -122,7 +120,7 @@ def add(lvol_id, snapshot_name):
             if sec_node.status == StorageNode.STATUS_ONLINE:
                 secondary_node = sec_node
             elif sec_node.status == StorageNode.STATUS_DOWN:
-                msg = f"Secondary node is in down status, can not create snapshot"
+                msg = "Secondary node is in down status, can not create snapshot"
                 logger.error(msg)
                 return False, msg
             else:
@@ -136,7 +134,7 @@ def add(lvol_id, snapshot_name):
 
         else:
             # both primary and secondary are not online
-            msg = f"Host nodes are not online"
+            msg = "Host nodes are not online"
             logger.error(msg)
             return False, msg
 
@@ -153,7 +151,7 @@ def add(lvol_id, snapshot_name):
             if snap_bdev:
                 snap_uuid = snap_bdev[0]['uuid']
                 blobid = snap_bdev[0]['driver_specific']['lvol']['blobid']
-                cluster_size = cluster.distr_ndcs * cluster.page_size_in_blocks
+                cluster_size = cluster.page_size_in_blocks
                 num_allocated_clusters = snap_bdev[0]["driver_specific"]["lvol"]["num_allocated_clusters"]
                 used_size = int(num_allocated_clusters*cluster_size)
             else:
@@ -189,6 +187,7 @@ def add(lvol_id, snapshot_name):
     snap.snap_bdev = f"{lvol.lvs_name}/{snap_bdev_name}"
     snap.created_at = int(time.time())
     snap.lvol = lvol
+    snap.vuid = snap_vuid
 
     snap.write_to_db(db_controller.kv_store)
 
@@ -234,18 +233,19 @@ def delete(snapshot_uuid, force_delete=False):
         logger.error(f"Snapshot not found {snapshot_uuid}")
         return False
 
-    snode = db_controller.get_storage_node_by_id(snap.lvol.node_id)
-    if not snode:
-        logger.error(f"Storage node not found {snap.lvol.node_id}")
+    try:
+        snode = db_controller.get_storage_node_by_id(snap.lvol.node_id)
+    except KeyError:
+        logger.exception(f"Storage node not found {snap.lvol.node_id}")
         return False
 
     clones = []
     for lvol in db_controller.get_lvols(snode.cluster_id):
-        if lvol.cloned_from_snap and lvol.cloned_from_snap == snapshot_uuid:
+        if lvol.cloned_from_snap and lvol.cloned_from_snap == snapshot_uuid and lvol.status != LVol.STATUS_IN_DELETION:
             clones.append(lvol)
 
     if len(clones) >= 1:
-        logger.warning(f"Soft delete snapshot with clones")
+        logger.warning("Soft delete snapshot with clones")
         snap = db_controller.get_snapshot_by_id(snapshot_uuid)
         snap.deleted = True
         snap.write_to_db(db_controller.kv_store)
@@ -282,7 +282,7 @@ def delete(snapshot_uuid, force_delete=False):
             if lvol_controller.is_node_leader(host_node, snap.lvol.lvs_name):
                 primary_node = host_node
                 if sec_node.status == StorageNode.STATUS_DOWN:
-                    msg = f"Secondary node is in down status, can not delete snapshot"
+                    msg = "Secondary node is in down status, can not delete snapshot"
                     logger.error(msg)
                     return False
 
@@ -312,7 +312,7 @@ def delete(snapshot_uuid, force_delete=False):
 
         else:
             # both primary and secondary are not online
-            msg = f"Host nodes are not online"
+            msg = "Host nodes are not online"
             logger.error(msg)
             return False
 
@@ -350,7 +350,7 @@ def delete(snapshot_uuid, force_delete=False):
     return True
 
 
-def clone(snapshot_id, clone_name, new_size=0):
+def clone(snapshot_id, clone_name, new_size=0, pvc_name=None, pvc_namespace=None):
     snap = db_controller.get_snapshot_by_id(snapshot_id)
     if not snap:
         msg=f"Snapshot not found {snapshot_id}"
@@ -368,10 +368,11 @@ def clone(snapshot_id, clone_name, new_size=0):
         logger.error(msg)
         return False, msg
 
-    snode = db_controller.get_storage_node_by_id(snap.lvol.node_id)
-    if not snode:
-        msg=f"Storage node not found: {snap.lvol.node_id}"
-        logger.error(msg)
+    try:
+        snode = db_controller.get_storage_node_by_id(snap.lvol.node_id)
+    except KeyError:
+        msg = 'Storage node not found'
+        logger.exception(msg)
         return False, msg
 
     cluster = db_controller.get_cluster_by_id(pool.cluster_id)
@@ -439,11 +440,16 @@ def clone(snapshot_id, clone_name, new_size=0):
     lvol.pool_uuid = pool.get_id()
     lvol.ha_type = snap.lvol.ha_type
     lvol.lvol_type = 'lvol'
-    lvol.guid = lvol_controller._generate_hex_string(16)
+    lvol.guid = utils.generate_hex_string(16)
     lvol.vuid = snap.lvol.vuid
     lvol.snapshot_name = snap.snap_bdev
     lvol.subsys_port = snap.lvol.subsys_port
 
+    if pvc_name:
+        lvol.pvc_name = pvc_name
+    if pvc_namespace:
+        lvol.namespace = pvc_namespace
+
     lvol.status = LVol.STATUS_ONLINE
     lvol.bdev_stack = [
         {
@@ -506,7 +512,7 @@ def clone(snapshot_id, clone_name, new_size=0):
             if lvol_controller.is_node_leader(host_node, lvol.lvs_name):
                 primary_node = host_node
                 if sec_node.status == StorageNode.STATUS_DOWN:
-                    msg = f"Secondary node is in down status, can not clone snapshot"
+                    msg = "Secondary node is in down status, can not clone snapshot"
                     logger.error(msg)
                     lvol.remove(db_controller.kv_store)
                     return False, msg
@@ -535,7 +541,7 @@ def clone(snapshot_id, clone_name, new_size=0):
 
         else:
             # both primary and secondary are not online
-            msg = f"Host nodes are not online"
+            msg = "Host nodes are not online"
             logger.error(msg)
             lvol.remove(db_controller.kv_store)
             return False, msg
