@@ -44,7 +44,7 @@ def deploy_mgmt_node(cluster_ip, cluster_id, ifname, cluster_secret):
         return False
 
     logger.info(f"Node IP: {DEV_IP}")
-    ret = scripts.configure_docker(DEV_IP)
+    scripts.configure_docker(DEV_IP)
 
     db_connection = cluster_data['db_connection']
     scripts.set_db_config(db_connection)
@@ -60,6 +60,10 @@ def deploy_mgmt_node(cluster_ip, cluster_id, ifname, cluster_secret):
             logger.error("Node already exists in the cluster")
             return False
 
+    if not cluster_data['disable_monitoring']:
+        utils.render_and_deploy_alerting_configs(cluster_data['contact_point'], cluster_data['grafana_endpoint'],
+                                                                    cluster_data['uuid'], cluster_data['secret'])
+
     logger.info("Joining docker swarm...")
     try:
         cluster_docker = utils.get_docker_client(cluster_id)
@@ -70,7 +74,7 @@ def deploy_mgmt_node(cluster_ip, cluster_id, ifname, cluster_secret):
             logger.info("Node is part of another swarm, leaving swarm")
             try:
                 cluster_docker.nodes.get(node_docker.info()["Swarm"]["NodeID"]).remove(force=True)
-            except:
+            except Exception:
                 pass
             node_docker.swarm.leave(force=True)
             time.sleep(5)
@@ -199,7 +203,7 @@ def remove_mgmt_node(uuid):
 
     logger.info("Leaving swarm...")
     node_docker = docker.DockerClient(base_url=f"tcp://{snode.docker_ip_port}", version="auto")
-    node_docker.swarm.leave()
+    node_docker.swarm.leave(force=True)
 
     mgmt_events.mgmt_remove(snode)
     logging.info("done")
