@@ -3,7 +3,8 @@ import pprint
 
 import json
 from inspect import ismethod
-from typing import Mapping
+import sys
+from typing import Mapping, Type
 from collections import ChainMap
 
 
@@ -26,10 +27,24 @@ class BaseModel(object):
         self.name = self.__class__.__name__
         self.from_dict(data)
 
-    def all_annotations(self) -> ChainMap:
+    @classmethod
+    def all_annotations(cls) -> Mapping[str, Type]:
         """Returns a dictionary-like ChainMap that includes annotations for all
            attributes defined in cls or inherited from superclasses."""
-        return ChainMap(*(c.__annotations__ for c in self.__class__.__mro__ if '__annotations__' in c.__dict__))
+        if sys.version_info >= (3, 10):
+            from inspect import get_annotations
+            return ChainMap(*(
+                get_annotations(c)
+                for c
+                in cls.__mro__
+            ))
+        else:
+            return ChainMap(*(
+                c.__annotations__
+                for c
+                in cls.__mro__
+                if '__annotations__' in c.__dict__
+            ))
 
     def get_id(self):
         return self.uuid
@@ -48,7 +63,7 @@ class BaseModel(object):
             return "%s/%s/%s" % (self.object_type, self.name, self.get_id())
 
     def from_dict(self, data):
-        for attr , value_dict in self.get_attrs_map().items():
+        for attr, value_dict in self.get_attrs_map().items():
             value = value_dict['default']
             if data is not None and attr in data:
                 dtype = value_dict['type']
@@ -56,12 +71,12 @@ class BaseModel(object):
                 if dtype in [int, float, str, bool]:
                     try:
                         value = dtype(value)
-                    except:
-                        if type(value) == list and dtype == int:
+                    except Exception:
+                        if type(value) is list and dtype is int:
                             value = len(value)
 
                 elif hasattr(dtype, '__origin__'):
-                    if dtype.__origin__ == list:
+                    if dtype.__origin__ is list:
                         if hasattr(dtype, "__args__") and hasattr(dtype.__args__[0], "from_dict"):
                             value = [dtype.__args__[0]().from_dict(item) for item in data[attr]]
                         else:
@@ -115,8 +130,10 @@ class BaseModel(object):
             for k, v in kv_store.get_range_startswith(prefix.strip().encode('utf-8'),  limit=limit, reverse=reverse):
                 objects.append(self.__class__().from_dict(json.loads(v)))
             return objects
-        except Exception as e:
-            print(f"Error reading from FDB: {e}")
+        except Exception:
+            from simplyblock_core import utils
+            logger = utils.get_logger(__name__)
+            logger.exception('Error reading from FDB')
             return []
 
     def get_last(self, kv_store):
