@@ -13,7 +13,7 @@ class CachedLVol(BaseModel):
     device_path: str = ""
     hostname: str = ""
     local_nqn: str = ""
-    lvol: LVol = None
+    lvol: LVol = None # type: ignore[assignment]
     lvol_id: str = ""
     ocf_bdev: str = ""
 
@@ -41,16 +41,16 @@ class CachingNode(BaseNodeObject):
     multipathing: bool = True
     node_lvs: str = "lvs"
     nvme_devices: List[NVMeDevice] = []
-    partitions_count: int = 0
+    partitions_count: int = 0  # Unused
     remote_devices: List[NVMeDevice] = []
     rpc_password: str = ""
     rpc_port: int = -1
     rpc_username: str = ""
-    sequential_number: int = 0
+    sequential_number: int = 0  # Unused
     services: List[str] = []
     subsystem: str = ""
     system_uuid: str = ""
     namespace: str = ""
     spdk_cpu_mask: str = ""
-    spdk_mem: int = ""
+    spdk_mem: int = 0
     spdk_image: str = ""
