@@ -102,13 +102,14 @@ class RPCClient:
     def subsystem_delete(self, nqn):
         return self._request("nvmf_delete_subsystem", params={'nqn': nqn})
 
-    def subsystem_create(self, nqn, serial_number, model_number, min_cntlid=1):
+    def subsystem_create(self, nqn, serial_number, model_number, min_cntlid=1, max_namespaces=32):
         params = {
             "nqn": nqn,
             "serial_number": serial_number,
             "allow_any_host": True,
             "min_cntlid": min_cntlid,
             "ana_reporting": True,
+            "max_namespaces": max_namespaces,
             "model_number": model_number}
         return self._request("nvmf_create_subsystem", params)
 
@@ -122,7 +123,7 @@ class RPCClient:
             params = {"trtype": trtype}
         return self._request("nvmf_get_transports", params)
 
-    def transport_create(self, trtype, qpair_count=6):
+    def transport_create(self, trtype, qpair_count=6,shared_bufs=24576):
         """
             [{'trtype': 'TCP', 'max_queue_depth': 128,
                'max_io_qpairs_per_ctrlr': 127, 'in_capsule_data_size': 4096,
@@ -137,20 +138,20 @@ class RPCClient:
         """
         params = {
             "trtype": trtype,
-            "max_io_qpairs_per_ctrlr": qpair_count,
-            "max_queue_depth": 64,
+            "max_io_qpairs_per_ctrlr": 128,
+            "max_queue_depth": 256,
             "abort_timeout_sec": 5,
-            "ack_timeout": 2048,
             "zcopy": True,
-            "in_capsule_data_size": 24500,
+            "in_capsule_data_size": 8192,
             "max_io_size": 131072,
             "io_unit_size": 8192,
             "max_aq_depth": 128,
-            "num_shared_buffers": 24576,
+            "num_shared_buffers": shared_bufs,
             "buf_cache_size": 512,
             "dif_insert_or_strip": False,
             "c2h_success": True,
-            "sock_priority": 0
+            "sock_priority": 0,
+            "ack_timeout": 4096,
         }
         return self._request("nvmf_create_transport", params)
 
@@ -219,7 +220,7 @@ class RPCClient:
         }
         return self._request2("ultra21_alloc_ns_init", params)
 
-    def nvmf_subsystem_add_ns(self, nqn, dev_name, uuid=None, nguid=None):
+    def nvmf_subsystem_add_ns(self, nqn, dev_name, uuid=None, nguid=None, nsid=None):
         params = {
             "nqn": nqn,
             "namespace": {
@@ -233,6 +234,9 @@ class RPCClient:
         if nguid:
             params['namespace']['nguid'] = nguid
 
+        if nsid:
+            params['namespace']['nsid'] = nsid
+
         return self._request("nvmf_subsystem_add_ns", params)
 
     def nvmf_subsystem_remove_ns(self, nqn, nsid):
@@ -393,7 +397,7 @@ class RPCClient:
 
     def bdev_alceml_create(self, alceml_name, nvme_name, uuid, pba_init_mode=3,
                            alceml_cpu_mask="", alceml_worker_cpu_mask="", pba_page_size=2097152,
-                           write_protection=False):
+                           write_protection=False, full_page_unmap=True):
         params = {
             "name": alceml_name,
             "cntr_path": nvme_name,
@@ -407,8 +411,7 @@ class RPCClient:
             "uuid": uuid,
             # "use_scheduling": True,
             "use_optimized": True,
-            "pba_nbalign": 4096,
-            "use_map_whole_page_on_1st_write": False
+            "pba_nbalign": 4096
         }
         if alceml_cpu_mask:
             params["bdb_lcpu_mask"] = int(alceml_cpu_mask, 16)
@@ -416,11 +419,14 @@ class RPCClient:
             params["bdb_lcpu_mask_alt_workers"] = int(alceml_worker_cpu_mask, 16)
         if write_protection:
             params["write_protection"] = True
+        if full_page_unmap:
+            params["use_map_whole_page_on_1st_write"] = True
         return self._request("bdev_alceml_create", params)
-
+       
     def bdev_distrib_create(self, name, vuid, ndcs, npcs, num_blocks, block_size, jm_names,
                             chunk_size, ha_comm_addrs=None, ha_inode_self=None, pba_page_size=2097152,
-                            distrib_cpu_mask="", ha_is_non_leader=True, jm_vuid=0, write_protection=False):
+                            distrib_cpu_mask="", ha_is_non_leader=True, jm_vuid=0, write_protection=False,
+                            full_page_unmap=True):
         """"
             // Optional (not specified = no HA)
             // Comma-separated communication addresses, for each node, e.g. "192.168.10.1:45001,192.168.10.1:32768".
@@ -436,7 +442,7 @@ class RPCClient:
             ret = self.get_bdevs(name)
             if ret:
                 return ret
-        except:
+        except Exception:
             pass
         params = {
             "name": name,
@@ -460,6 +466,8 @@ class RPCClient:
             params["bdb_lcpu_mask"] = int(distrib_cpu_mask, 16)
         if write_protection:
             params["write_protection"] = True
+        if full_page_unmap:
+            params["use_map_whole_page_on_1st_write"] = True
         return self._request("bdev_distrib_create", params)
 
     def bdev_lvol_delete_lvstore(self, name):
@@ -474,8 +482,10 @@ class RPCClient:
         params = {"name": name}
         return self._request2("bdev_alceml_delete", params)
 
-    def get_lvol_stats(self, uuid):
-        params = {"name": uuid}
+    def get_lvol_stats(self, uuid=""):
+        params = {}
+        if uuid:
+            params["uuid"] = uuid
         return self._request("bdev_get_iostat", params)
 
     def bdev_raid_create(self, name, bdevs_list, raid_level="0", strip_size_kb=4):
@@ -483,7 +493,7 @@ class RPCClient:
             ret = self.get_bdevs(name)
             if ret:
                 return ret
-        except:
+        except Exception:
             pass
         params = {
             "name": name,
@@ -516,6 +526,25 @@ class RPCClient:
             params['w_mbytes_per_sec'] = w_mbytes_per_sec
         return self._request("bdev_set_qos_limit", params)
 
+    def bdev_lvol_add_to_group(self, group_id, lvol_name_list):
+        return True
+        # params = {
+        #     "bdev_group_id": group_id ,
+        #     "lvol_vbdev_list": lvol_name_list
+        # }
+        # return self._request("bdev_lvol_add_to_group", params)
+
+    def bdev_lvol_set_qos_limit(self, bdev_group_id, rw_ios_per_sec, rw_mbytes_per_sec, r_mbytes_per_sec, w_mbytes_per_sec):
+        return True
+        # params = {
+        #     "bdev_group_id": bdev_group_id,
+        #     "rw_ios_per_sec": rw_ios_per_sec,
+        #     "rw_mbytes_per_sec": rw_mbytes_per_sec,
+        #     "r_mbytes_per_sec": r_mbytes_per_sec,
+        #     "w_mbytes_per_sec": w_mbytes_per_sec
+        # }
+        # return self._request("bdev_lvol_set_qos_limit", params)
+
     def distr_send_cluster_map(self, params):
         return self._request("distr_send_cluster_map", params)
 
@@ -533,7 +562,7 @@ class RPCClient:
         # ultra/DISTR_v2/src_code_app_spdk/specs/message_format_rpcs__distrib__v5.txt#L396C1-L396C27
         return self._request("distr_status_events_update", params)
 
-    def bdev_nvme_attach_controller_tcp(self, name, nqn, ip, port):
+    def bdev_nvme_attach_controller_tcp(self, name, nqn, ip, port, multipath=False):
         params = {
             "name": name,
             "trtype": "tcp",
@@ -542,12 +571,12 @@ class RPCClient:
             "trsvcid": str(port),
             "subnqn": nqn,
             "fabrics_connect_timeout_us": 100000,
-            # "fast_io_fail_timeout_sec": 1,
-            "num_io_queues": 16384,
-            # "ctrlr_loss_timeout_sec": 1,
-            "multipath":"disable",
-            # "reconnect_delay_sec":1
+            "num_io_queues": 128,
         }
+        if multipath:
+            params["multipath"] = "failover"
+        else:
+            params["multipath"] = "disable"
         return self._request("bdev_nvme_attach_controller", params)
 
     def bdev_nvme_attach_controller_tcp_caching(self, name, nqn, ip, port):
@@ -604,14 +633,14 @@ class RPCClient:
     def bdev_nvme_set_options(self):
         params = {
             # "action_on_timeout": "abort",
-            "bdev_retry_count": 0,
+            "bdev_retry_count": 1,
             "transport_retry_count": 3,
             "ctrlr_loss_timeout_sec": 1,
-            "fast_io_fail_timeout_sec": 0,
+            "fast_io_fail_timeout_sec" : 0,
             "reconnect_delay_sec": 1,
-            "keep_alive_timeout_ms": 10000,
-            "transport_ack_timeout": 10,
-            "timeout_us": constants.NVME_TIMEOUT_US
+            "keep_alive_timeout_ms": 30000,
+            "timeout_us": constants.NVME_TIMEOUT_US,
+            "transport_ack_timeout": 12,
         }
         return self._request("bdev_nvme_set_options", params)
 
@@ -911,18 +940,12 @@ class RPCClient:
             params = {"jm_vuid": jm_vuid}
         return self._request("bdev_distrib_force_to_non_leader", params)
 
-    def bdev_lvol_set_leader(self, is_leader=False, uuid=None, lvs_name=None, bs_nonleadership=False):
-        params = {
-            "lvs_leadership": is_leader,
-        }
-        if uuid:
-            params["uuid"] = uuid
-        elif lvs_name:
-            params["lvs_name"] = lvs_name
-
-        params["bs_nonleadership"] = bs_nonleadership
-
-        return self._request("bdev_lvol_set_leader_all", params)
+    def bdev_lvol_set_leader(self, lvs, *, leader=False, bs_nonleadership=False):
+        return self._request("bdev_lvol_set_leader_all", {
+            "uuid" if utils.UUID_PATTERN.match(lvs) else "lvs_name": lvs,
+            "lvs_leadership": leader,
+            "bs_nonleadership": bs_nonleadership,
+        })
 
     def bdev_lvol_register(self, name, lvs_name, registered_uuid, blobid, priority_class=0):
         params = {
@@ -980,13 +1003,19 @@ class RPCClient:
         }
         return self._request("nvmf_set_max_subsystems", params)
 
-    def bdev_lvol_set_lvs_ops(self, lvs_name, groupid, subsystem_port=9090):
-        params = {
+    def bdev_lvol_set_lvs_opts(self, lvs, *, groupid, subsystem_port=9090, primary=False, secondary=False):
+        """Set lvstore options
+
+        `lvs` must be either an ID or the lvstore name.
+        """
+
+        return self._request('bdev_lvol_set_lvs_opts', {
+            "uuid" if utils.UUID_PATTERN.match(lvs) else "lvs_name": lvs,
             "groupid": groupid,
-            "lvs_name": lvs_name,
             "subsystem_port": subsystem_port,
-        }
-        return self._request("bdev_lvol_set_lvs_op", params)
+            "primary": primary,
+            "secondary": secondary,
+        })
 
     def bdev_lvol_get_lvol_delete_status(self, name):
         """
@@ -1008,3 +1037,46 @@ class RPCClient:
         }
         return self._request("bdev_lvol_set_lvs_read_only", params)
 
+    def bdev_lvol_create_hublvol(self, lvs):
+        return self._request('bdev_lvol_create_hublvol', {
+            "uuid" if utils.UUID_PATTERN.match(lvs) else "lvs_name": lvs,
+        })
+
+    def bdev_lvol_delete_hublvol(self, lvs):
+        return self._request('bdev_lvol_delete_hublvol', {
+            "uuid" if utils.UUID_PATTERN.match(lvs) else "lvs_name": lvs,
+        })
+
+    def bdev_lvol_connect_hublvol(self, lvs, bdev):
+        return self._request('bdev_lvol_connect_hublvol', {
+            "uuid" if utils.UUID_PATTERN.match(lvs) else "lvs_name": lvs,
+            "remote_bdev": bdev,
+        })
+
+    def jc_suspend_compression(self, jm_vuid, suspend=False):
+        params = {
+            "jm_vuid": jm_vuid,
+            "suspend": suspend,
+        }
+        return self._request("jc_suspend_compression", params)
+
+    def nvmf_subsystem_add_listener(self, nqn, trtype, traddr, trsvcid, ana_state=None):
+        params = {
+            "nqn": nqn,
+            "listen_address": {
+                "trtype": trtype,
+                "adrfam": "IPv4",
+                "traddr": traddr,
+                "trsvcid": str(trsvcid)
+            }
+        }
+        if ana_state:
+            params["ana_state"] = ana_state
+        return self._request2("nvmf_subsystem_add_listener", params)
+
+    def bdev_nvme_set_multipath_policy(self, name, policy):  # policy: active_active or active_passive
+        params = {
+            "name": name,
+            "policy": policy,
+        }
+        return self._request("bdev_nvme_set_multipath_policy", params)
