@@ -9,12 +9,12 @@ from flask import Blueprint, request
 from simplyblock_core.controllers import tasks_controller
 from simplyblock_web import utils
 
-from simplyblock_core import db_controller, storage_node_ops, utils as core_utils
+from simplyblock_core import db_controller, storage_node_ops
 
 logger = logging.getLogger(__name__)
 
 bp = Blueprint("snode", __name__)
-db_controller = db_controller.DBController()
+db = db_controller.DBController()
 
 
 @bp.route('/storagenode', methods=['GET'], defaults={'uuid': None})
@@ -22,18 +22,21 @@ db_controller = db_controller.DBController()
 def list_storage_nodes(uuid):
     cluster_id = utils.get_cluster_id(request)
     if uuid:
-        node = db_controller.get_storage_node_by_id(uuid)
-        if node and node.cluster_id == cluster_id:
-            nodes = [node]
-        else:
+        try:
+            node = db.get_storage_node_by_id(uuid)
+            if node.cluster_id == cluster_id:
+                nodes = [node]
+            else:
+                return utils.get_response_error(f"node not found: {uuid}", 404)
+        except KeyError:
             return utils.get_response_error(f"node not found: {uuid}", 404)
     else:
-        nodes = db_controller.get_storage_nodes_by_cluster_id(cluster_id)
+        nodes = db.get_storage_nodes_by_cluster_id(cluster_id)
     data = []
     for node in nodes:
         d = node.get_clean_dict()
         d['status_code'] = node.get_status_code()
-        lvs = db_controller.get_lvols_by_node_id(node.get_id()) or []
+        lvs = db.get_lvols_by_node_id(node.get_id()) or []
         d['lvols'] = len(lvs)
         data.append(d)
     return utils.get_response(data)
@@ -42,8 +45,9 @@ def list_storage_nodes(uuid):
 @bp.route('/storagenode/capacity/<string:uuid>/history/<string:history>', methods=['GET'])
 @bp.route('/storagenode/capacity/<string:uuid>', methods=['GET'], defaults={'history': None})
 def storage_node_capacity(uuid, history):
-    node = db_controller.get_storage_node_by_id(uuid)
-    if not node:
+    try:
+        db.get_storage_node_by_id(uuid)
+    except KeyError:
         return utils.get_response_error(f"node not found: {uuid}", 404)
 
     data = storage_node_ops.get_node_capacity(uuid, history, parse_sizes=False)
@@ -57,8 +61,9 @@ def storage_node_capacity(uuid, history):
 @bp.route('/storagenode/iostats/<string:uuid>/history/<string:history>', methods=['GET'])
 @bp.route('/storagenode/iostats/<string:uuid>', methods=['GET'], defaults={'history': None})
 def storagenode_iostats(uuid, history):
-    node = db_controller.get_storage_node_by_id(uuid)
-    if not node:
+    try:
+        node = db.get_storage_node_by_id(uuid)
+    except KeyError:
         return utils.get_response_error(f"node not found: {uuid}", 404)
 
     data = storage_node_ops.get_node_iostats_history(uuid, history, parse_sizes=False, with_sizes=True)
@@ -71,8 +76,9 @@ def storagenode_iostats(uuid, history):
 
 @bp.route('/storagenode/port/<string:uuid>', methods=['GET'])
 def storage_node_ports(uuid):
-    node = db_controller.get_storage_node_by_id(uuid)
-    if not node:
+    try:
+        node = db.get_storage_node_by_id(uuid)
+    except KeyError:
         return utils.get_response_error(f"node not found: {uuid}", 404)
 
     out = []
@@ -89,7 +95,7 @@ def storage_node_ports(uuid):
 
 @bp.route('/storagenode/port-io-stats/<string:uuid>', methods=['GET'])
 def storage_node_port_io_stats(uuid):
-    nodes = db_controller.get_storage_nodes()
+    nodes = db.get_storage_nodes()
     nd = None
     port = None
     for node in nodes:
@@ -102,7 +108,7 @@ def storage_node_port_io_stats(uuid):
     if not port:
         return utils.get_response_error(f"Port not found: {uuid}", 404)
 
-    data = db_controller.get_port_stats(nd.get_id(), port.get_id())
+    data = db.get_port_stats(nd.get_id(), port.get_id())
     out = []
     for record in data:
         out.append(record.get_clean_dict())
@@ -111,8 +117,9 @@ def storage_node_port_io_stats(uuid):
 
 @bp.route('/storagenode/suspend/<string:uuid>', methods=['GET'])
 def storage_node_suspend(uuid):
-    node = db_controller.get_storage_node_by_id(uuid)
-    if not node:
+    try:
+        db.get_storage_node_by_id(uuid)
+    except KeyError:
         return utils.get_response_error(f"node not found: {uuid}", 404)
 
     out = storage_node_ops.suspend_storage_node(uuid, True)
@@ -121,8 +128,9 @@ def storage_node_suspend(uuid):
 
 @bp.route('/storagenode/resume/<string:uuid>', methods=['GET'])
 def storage_node_resume(uuid):
-    node = db_controller.get_storage_node_by_id(uuid)
-    if not node:
+    try:
+        db.get_storage_node_by_id(uuid)
+    except KeyError:
         return utils.get_response_error(f"node not found: {uuid}", 404)
 
     out = storage_node_ops.resume_storage_node(uuid)
@@ -131,15 +139,16 @@ def storage_node_resume(uuid):
 
 @bp.route('/storagenode/shutdown/<string:uuid>', methods=['GET'])
 def storage_node_shutdown(uuid):
-    node = db_controller.get_storage_node_by_id(uuid)
-    if not node:
+    try:
+        db.get_storage_node_by_id(uuid)
+    except KeyError:
         return utils.get_response_error(f"node not found: {uuid}", 404)
 
     force = True
     try:
         args = request.args
         force = bool(args.get('force', True))
-    except:
+    except Exception:
         pass
 
     threading.Thread(
@@ -155,10 +164,12 @@ def storage_node_restart():
     req_data = request.get_json()
     uuid = req_data.get("uuid", "")
     node_ip = req_data.get("node_ip", "")
+    reattach_volume = bool(req_data.get("reattach_volume", False))
     force = bool(req_data.get("force", ""))
 
-    node = db_controller.get_storage_node_by_id(uuid)
-    if not node:
+    try:
+        db.get_storage_node_by_id(uuid)
+    except KeyError:
         return utils.get_response_error(f"node not found: {uuid}", 404)
 
     threading.Thread(
@@ -167,6 +178,7 @@ def storage_node_restart():
             "node_id": uuid,
             "node_ip": node_ip,
             "force": force,
+            "reattach_volume": reattach_volume,
         }
     ).start()
 
@@ -180,25 +192,17 @@ def storage_node_add():
     if 'cluster_id' not in req_data:
         return utils.get_response_error("missing required param: cluster_id", 400)
 
-    if 'node_ip' not in req_data:
-        return utils.get_response_error("missing required param: node_ip", 400)
+    if 'node_addr' not in req_data:
+        return utils.get_response_error("missing required param: node_addr", 400)
 
     if 'ifname' not in req_data:
         return utils.get_response_error("missing required param: ifname", 400)
 
-    if 'max_lvol' not in req_data:
-        return utils.get_response_error("missing required param: max_lvol", 400)
-
-    if 'max_prov' not in req_data:
-        return utils.get_response_error("missing required param: max_prov", 400)
-
     cluster_id = req_data['cluster_id']
-    node_ip = req_data['node_ip']
+    node_addr = req_data['node_addr']
     ifname = req_data['ifname']
-    max_lvol = int(req_data['max_lvol'])
     max_snap = int(req_data.get('max_snap', 500))
-    max_prov = core_utils.parse_size(req_data['max_prov'], unit='G')
-    number_of_distribs = int(req_data.get('number_of_distribs', 2))
+
     if req_data.get('disable_ha_jm', "") == "true":
         disable_ha_jm = True
     else:
@@ -207,9 +211,9 @@ def storage_node_add():
     enable_test_device = False
     param = req_data.get('enable_test_device')
     if param:
-        if type(param) == bool:
+        if isinstance(param, bool):
             enable_test_device = param
-        elif type(param) == str:
+        elif isinstance(param, str):
             enable_test_device = param == "true"
 
     spdk_image = None
@@ -237,18 +241,6 @@ def storage_node_add():
     if 'partitions' in req_data:
         partitions = int(req_data['partitions'])
 
-    number_of_devices = 0
-    if 'number_of_devices' in req_data:
-        number_of_devices = int(req_data['number_of_devices'])
-
-    spdk_cpu_mask = None
-    if 'spdk_cpu_mask' in req_data:
-        msk = req_data['spdk_cpu_mask']
-        if utils.validate_cpu_mask(msk):
-            spdk_cpu_mask = msk
-        else:
-            return utils.get_response_error(f"Invalid cpu mask value: {msk}", 400)
-
     iobuf_small_pool_count = 0
     if 'iobuf_small_pool_count' in req_data:
         iobuf_small_pool_count = int(req_data['iobuf_small_pool_count'])
@@ -257,37 +249,21 @@ def storage_node_add():
     if 'iobuf_large_pool_count' in req_data:
         iobuf_large_pool_count = int(req_data['iobuf_large_pool_count'])
 
-    is_secondary_node = False
-    if 'is_secondary_node' in req_data:
-        is_secondary_node = bool(req_data['is_secondary_node'])
-
-    ssd_pcie = []
-    if 'ssd_pcie' in req_data:
-        ssd_pcie = req_data['ssd_pcie']
-
     tasks_controller.add_node_add_task(cluster_id, {
         "cluster_id": cluster_id,
-        "node_ip": node_ip,
+        "node_addr": node_addr,
         "iface_name": ifname,
         "data_nics_list": data_nics,
-        "max_lvol": max_lvol,
         "max_snap": max_snap,
-        "max_prov": max_prov,
-        "spdk_cpu_mask": spdk_cpu_mask,
         "spdk_image": spdk_image,
         "spdk_debug": spdk_debug,
         "small_bufsize": iobuf_small_pool_count,
         "large_bufsize": iobuf_large_pool_count,
         "num_partitions_per_dev": partitions,
         "jm_percent": jm_percent,
-        "number_of_devices": number_of_devices,
         "enable_test_device": enable_test_device,
-        "number_of_distribs": number_of_distribs,
         "namespace": namespace,
         "enable_ha_jm": not disable_ha_jm,
-        "is_secondary_node": is_secondary_node,
-        "ssd_pcie": ssd_pcie,
-
     })
 
     return utils.get_response(True)
@@ -295,8 +271,9 @@ def storage_node_add():
 
 @bp.route('/storagenode/make-sec-new-primary/<string:uuid>', methods=['GET'])
 def make_primary(uuid):
-    node = db_controller.get_storage_node_by_id(uuid)
-    if not node:
+    try:
+        db.get_storage_node_by_id(uuid)
+    except KeyError:
         return utils.get_response_error(f"node not found: {uuid}", 404)
 
     out = storage_node_ops.make_sec_new_primary(uuid)
